<?php

namespace TDevAgency\Cms\Controllers\Admin;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\View;
use TDevAgency\Cms\Controllers\Controller;
use TDevAgency\Cms\Models\BaseModels\Post;
use TDevAgency\Cms\Models\BaseModels\PostConnection;
use TDevAgency\Cms\Models\BaseModels\PostLocalization;
use TDevAgency\Cms\Traits\CategoryTrait;
use TDevAgency\Cms\Traits\PostHelper;

class CategoriesController extends Controller
{
    use CategoryTrait;
    use PostHelper;

    /**
     * @return \Illuminate\Contracts\View\View
     */
    public function index(Request $request)
    {
        return $this->displayList(
            Post::CATEGORY_TYPE,
            $request,
            ['index' => 'cms.categories.index', 'edit' => 'cms.categories.edit', 'destroy' => 'cms.categories.destroy']
        );
    }

    /**
     * @return \Illuminate\Contracts\View\View
     */
    public function create()
    {
        $categories = Post::where('post_type', Post::CATEGORY_TYPE)->with(['adminLocalization'])->get();
        return View::make('cms::admin.screens.categories.single')
            ->with('post', new Post())
            ->with('categories', $categories)
            ->with('selectedCategories', []);
    }

    /**
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function store(Request $request)
    {
        $this->validate($request, $this->validatorsArray());

        $category = new Post();
        try {
            DB::transaction(function () use ($request, &$category) {
                $this->postPrepare($request, $category);
                $category->post_type = Post::CATEGORY_TYPE;
                $category->save();

                if ($request->has('localizations')) {
                    $this->saveLocalization($request->get('localizations'), $category);
                }

                if ($request->has('category')) {
                    foreach ($request->get('category') as $id => $categories) {
                        $postConnection = new PostConnection();
                        $postConnection->post_parent_id = $id;
                        $postConnection->post_id = $category->id;
                        $postConnection->save();
                    }
                }
            });

            return redirect()->route('cms.categories.index')->with('toast', [
                'title' => trans('cms::admin.toast.created'),
                'status' => 'success'
            ]);
        } catch (\Throwable $e) {
            Log::error($e);
            return redirect()->back()->withInput()->with('toast', [
                'title' => trans('cms::admin.toast.not_created'),
                'status' => 'danger'
            ]);
        }
    }

    /**
     * @param $id
     */
    public function show($id)
    {
        //
    }

    /**
     * @param Post $category
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function edit($category)
    {
        [$categories, $selectedCategories] = $this->categories($category->id);
        $category->setRelation('localizations', $category->localizations->mapWithKeys(function (PostLocalization $postLocalization) {
            return [$postLocalization->lang => $postLocalization];
        }));

        return view('cms::admin.screens.categories.single')
            ->with('post', $category)
            ->with('categories', $categories)
            ->with('selectedCategories', $selectedCategories);
    }

    /**
     * @param Request $request
     * @param Post $category
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function update(Request $request, $category)
    {
        $this->validate($request, $this->validatorsArray($category->id));
        try {
            DB::transaction(function () use ($request, &$category) {
                $this->postPrepare($request, $category);
                $category->update();

                if ($request->has('localizations')) {
                    $this->saveLocalization($request->get('localizations'), $category);
                }

                if ($request->has('category')) {
                    PostConnection::where('post_id', $category->id)->whereNotIn('post_parent_id', $request->get('category'))->delete();
                    foreach ($request->get('category') as $key => $value) {
                        PostConnection::updateOrCreate(
                            [
                                'post_id' => $category->id,
                                'post_parent_id' => $key,
                            ],
                            [
                                'post_id' => $category->id,
                                'post_parent_id' => $key,
                            ]
                        );
                    }
                }
            });
            return redirect()->route('cms.categories.index')->with('toast', [
                'title' => trans('cms::admin.toast.updated'),
                'status' => 'success'
            ]);
        } catch (\Throwable $e) {
            Log::error($e);
            return redirect()->back()->withInput()->with('toast', [
                'title' => trans('cms::admin.toast.not_updated'),
                'status' => 'danger'
            ]);
        }
    }

    /**
     * @param Post $category
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Throwable
     */
    public function destroy($category)
    {
        DB::transaction(function () use ($category) {
            PostConnection::where('post_id', $category->id)->delete();
            PostConnection::where('post_parent_id', $category->id)->delete();
            PostLocalization::where('post_id', $category->id)->delete();
            Post::where('id', $category->id)->delete();
        });
        return back()->with('toast', [
            'title' => trans('cms::admin.toast.deleted'),
            'status' => 'success'
        ]);
    }
}
